Index: asterisk/channels/chan_sip.c
===================================================================
RCS file: /usr/cvsroot/asterisk/channels/chan_sip.c,v
retrieving revision 1.703
diff -u -3 -r1.703 chan_sip.c
--- asterisk/channels/chan_sip.c	6 Apr 2005 21:12:32 -0000	1.703
+++ asterisk/channels/chan_sip.c	8 Apr 2005 17:06:06 -0000
@@ -2633,6 +2633,7 @@
 static int sip_register(char *value, int lineno)
 {
 	struct sip_registry *reg;
+	struct sip_registry* found = NULL;
 	char copy[256] = "";
 	char *username=NULL, *hostname=NULL, *secret=NULL, *authuser=NULL;
 	char *porta=NULL;
@@ -2694,7 +2695,29 @@
 		reg->portno = porta ? atoi(porta) : 0;
 		reg->callid_valid = 0;
 		reg->ocseq = 101;
-		ASTOBJ_CONTAINER_LINK(&regl, reg);
+
+		ASTOBJ_CONTAINER_TRAVERSE(&regl, !found, do {
+			ASTOBJ_WRLOCK(iterator);
+			if ((iterator->objflags & ASTOBJ_FLAG_MARKED) != 0) {
+				if ((strcmp(reg->username, iterator->username) == 0) &&
+				    (strcmp(reg->hostname, iterator->hostname) == 0) &&
+				    (strcmp(reg->authuser, iterator->authuser) == 0) &&
+				    (strcmp(reg->secret, iterator->secret) == 0) &&
+				    (strcmp(reg->contact, iterator->contact) == 0) &&
+				    (reg->portno == iterator->portno))
+				{
+					found = iterator;
+					iterator->refresh = default_expiry;
+					ASTOBJ_UNMARK(iterator);
+				}
+			}
+			ASTOBJ_UNLOCK(iterator);
+    		} while (0)
+		);
+
+		if (!found)
+			ASTOBJ_CONTAINER_LINK(&regl, reg);
+	
 		ASTOBJ_UNREF(reg,sip_registry_destroy);
 	} else {
 		ast_log(LOG_ERROR, "Out of memory\n");
@@ -10733,7 +10756,6 @@
 {
 	/* Delete all users */
 	ASTOBJ_CONTAINER_DESTROYALL(&userl,sip_destroy_user);
-	ASTOBJ_CONTAINER_DESTROYALL(&regl,sip_registry_destroy);
 	ASTOBJ_CONTAINER_MARKALL(&peerl);
 }
 
@@ -10761,10 +10783,10 @@
 	int ms;
 	ASTOBJ_CONTAINER_TRAVERSE(&regl, 1, do {
 		ASTOBJ_WRLOCK(iterator);
-		if (iterator->expire > -1)
-			ast_sched_del(sched, iterator->expire);
-		ms = (rand() >> 12) & 0x1fff;
-		iterator->expire = ast_sched_add(sched, ms, sip_reregister, iterator);
+		if (iterator->expire == -1) {
+			ms = (rand() >> 12) & 0x1fff;
+			iterator->expire = ast_sched_add(sched, ms, sip_reregister, iterator);
+		}
 		ASTOBJ_UNLOCK(iterator);
 	} while (0)
 	);
@@ -10777,8 +10799,10 @@
 	authl = (struct sip_auth *) NULL;
 
 	delete_users();
+	ASTOBJ_CONTAINER_MARKALL(&regl);
 	reload_config();
 	prune_peers();
+	ASTOBJ_CONTAINER_PRUNE_MARKED(&regl, sip_registry_destroy);
 	sip_poke_all_peers();
 	sip_send_all_registers();
 
